const Overview = imports.ui.main.overview;
const GLib = imports.gi.GLib;

const ViewSelector = imports.ui.viewSelector;

const Main = imports.ui.main;

let _signal = [];
let _function;

let _manager, _workspace, _monitor;

var _showAppsButtonChecked;

function hideApps()
{
    // hide the overview only if we're in application view
    if (Overview.viewSelector._showAppsButton.checked)
        Overview.hide();
}

function showApps()
{
    if (_workspace.list_windows()./*filter(window => window.get_monitor() == _monitor).*/length == 0)
        Overview.viewSelector.showApps();
}

function disconnectWindowSignals()
{
    if (_signal['window-added'])
        _workspace.disconnect(_signal['window-added']);

    if (_signal['window-removed'])
        _workspace.disconnect(_signal['window-removed']);
}

function getWorkspace()
{
    _workspace = _manager.get_active_workspace();

    _signal['window-added'] = _workspace.connect('window-added', hideApps);
    _signal['window-removed'] = _workspace.connect('window-removed', showApps);
}

function checkWorkspace()
{
    disconnectWindowSignals();

    getWorkspace();

    if (!Overview.visible)
        showApps();
    else
        hideApps();
}

function overviewHidden()
{
    // don't show the apps view if we were already looking at it
    if (!_showAppsButtonChecked)
        showApps();
}

function animateFromOverview()
{
    // the original function sets _showAppsButton.checked = false, so we need to copy it to a local variable first
    _showAppsButtonChecked = Overview.viewSelector._showAppsButton.checked;
    _function.apply(this);
}

function init()
{
    _manager = global.screen;
    if (_manager == undefined)
        _manager = global.workspace_manager;

    _monitor = global.display.get_primary_monitor();
}

function enable()
{
    _function = ViewSelector.ViewSelector.prototype.animateFromOverview;
    ViewSelector.ViewSelector.prototype.animateFromOverview = animateFromOverview;

    getWorkspace();

    _signal['workspace-switched'] = _manager.connect('workspace-switched', checkWorkspace);
    _signal['overview-hidden'] = Overview.connect('hidden', overviewHidden);

    // shows applications at login, but waits for mainloop to turn idle first
    // .. and then waits 1 second
    GLib.idle_add(GLib.PRIORITY_LOW, () => {
        GLib.timeout_add_seconds(GLib.PRIORITY_LOW, 1, () => {
            showApps();
            return GLib.SOURCE_REMOVE;
        });
        return GLib.SOURCE_REMOVE;
    });
}

function disable()
{
    disconnectWindowSignals();

    Overview.disconnect(_signal['overview-hidden']);
    _manager.disconnect(_signal['workspace-switched']);

    ViewSelector.ViewSelector.prototype.animateFromOverview = _function;
}
